#!/usr/bin/env python
##
## This file is part of the OpenSIPS Python Package
## (see https://github.com/OpenSIPS/python-opensips).
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.
##


""" Module that implements OpenSIPS Event behavior """

from threading import Thread, Event
from ..mi import OpenSIPSMI, OpenSIPSMIException
from .datagram import Datagram
from .stream import Stream

class OpenSIPSEventException(Exception):
    """ Exceptions generated by OpenSIPS Events """

class OpenSIPSEvent():

    """ Implementation of the OpenSIPS Event """

    def __init__(self, mi: OpenSIPSMI, _type: str, **kwargs):
        self.mi = mi
        self.kwargs = kwargs
        self.thread = None
        self.thread_stop = Event()
        self.thread_stop.clear()

        if _type == "datagram":
            self.socket = Datagram(**kwargs)
        elif _type == "stream":
            self.socket = Stream(**kwargs)
        else:
            raise ValueError("Invalid event type")

    def handle(self, callback):
        """ Handles the event callbacks """
        while not self.thread_stop.is_set():
            data = self.socket.read()
            if data:
                callback(data)

    def subscribe(self, event: str, callback, expire=None):
        """ Subscribes for an event """
        try:
            sock_name = self.socket.create()
            if expire is None:
                ret_val = self.mi.execute("event_subscribe", [event, sock_name])
            else:
                ret_val = self.mi.execute("event_subscribe", [event, sock_name, expire])

            if ret_val != "OK":
                raise OpenSIPSEventException("Failed to subscribe to event")

            self.thread = Thread(target=self.handle, args=(callback,))
            self.thread.start()

        except OpenSIPSMIException as e:
            raise e
        except Exception as e:
            raise OpenSIPSEventException(f"Failed to subscribe to event: {e}") from e

    def unsubscribe(self, event: str):
        """ Unsubscribes for an event """
        try:
            ret_val = self.mi.execute("event_subscribe", [event, self.socket.sock_name, 0])

            if ret_val != "OK":
                raise OpenSIPSEventException("Failed to unsubscribe from event")

            self.stop()

        except OpenSIPSMIException as e:
            raise e

    def stop(self):
        """ Stops the current event processing """
        self.thread_stop.set()
        self.thread.join()
        self.socket.destroy()
